/*
 *  - common/fs.c
 *
 *  - file system access functions
 *
 *  This file is part of the VC 1541 Emulator.
 *  Copyright (c) 1997 Torsten Paul
 */

#include <stdio.h>
#include <dirent.h>

#include "fileio.h"

extern int timeout; 			           /* this is set by Wait */

static int fs_check_type(char *path, struct stat *st)
{
	if (st->st_mode & S_IFDIR) return 1;
	return 0;
}

static int fs_send_dir(device_t *d)
{
	int a, b, c;
	char *type;
	DIR *dir;
	struct dirent *ent;
	struct stat st;
	unsigned char buf[1024];
	unsigned char path[1024];
	unsigned int addr, len, blocks;

	if ((dir = opendir(d->path)) == NULL) {
		return 0;
	}
	DPRINT("!");
	addr = 0x0801;
	PutIECByte(addr, 0); if (timeout) return 0;
	PutIECByte(addr >> 8, 0); if (timeout) return 0;
	sprintf(buf, "\x12\"%s\"", d->path);
	len = strlen(buf);
	addr += len + 5;
	PutIECByte(addr, 0); if (timeout) return 0;
	PutIECByte(addr >> 8, 0); if (timeout) return 0;
	PutIECByte(0x00, 0); if (timeout) return 0;
	PutIECByte(0x00, 0); if (timeout) return 0;
	for (a = 0;a < len;a++) {
		c = toupper(buf[a]);
		if (c == '\\') c = '/';

		PutIECByte(c, 0); if (timeout) return 0;
	}
	PutIECByte(0x00, 0); if (timeout) return 0;

	while ((ent = readdir(dir)) != NULL) {
		a = strlen(d->path);
		if (a > 0 && d->path[a - 1] == '\\') {
			sprintf(path, "%s%s", d->path, ent->d_name);
		} else {
			sprintf(path, "%s\\%s", d->path, ent->d_name);
		}
		sprintf(buf, "\"%s\"", ent->d_name);
		if (stat(path, &st) == -1) {
			type = "???";
		} else {
			if (st.st_mode & S_IFDIR) {
				type = "DIR";
			} else {
				type = "PRG";
			}
		}
		blocks = (st.st_size + 253) / 254;
		b = 1;
		if (blocks < 100) b++;
		if (blocks < 10) b++;
		len = strlen(buf);
		addr += 26 + b;
		PutIECByte(addr, 0); if (timeout) return 0;
		PutIECByte(addr >> 8, 0); if (timeout) return 0;
		PutIECByte(blocks, 0); if (timeout) return 0;
		PutIECByte(blocks >> 8, 0); if (timeout) return 0;
		for (a = 0;a < b;a++) {
			PutIECByte(' ', 0);
			if (timeout) return 0;
		}
		for (a = 0;a < len;a++) {
			PutIECByte(buf[a], 0);
			if (timeout) return 0;
		}
		for (a = 0;a < 18 - len;a++) {
			PutIECByte(' ', 0);
			if (timeout) return 0;
		}
		for (a = 0;a < 3;a++) {
			PutIECByte(type[a], 0);
			if (timeout) return 0;
		}
		PutIECByte(0x00, 0);
		if (timeout) return 0;
	}
	closedir(dir);
	PutIECByte(0x00, 0); if (timeout) return 0;
	PutIECByte(0x00, 1); if (timeout) return 0;

	return 1;
}

static int fs_send_file(device_t *d, char *filename)
{
	FILE *f;
	byte_t c1;
	int a, c2;
	struct stat st;
	word_t b, c, blocks;
	char path[MAXPATH], *fname;

	util_glob_dir(path, d->path, filename);

	if ((f = fopen(path, "rb")) == NULL) return 0;

	if ((c2 = fgetc(f)) == EOF) return 0;

	fstat(fileno(f), &st);
	blocks = (st.st_size + 253) / 254;

	b = c = 0;
	fname = strrchr(path, '\\');
	fname++;
	do {
		c1 = (byte_t)c2;
		c2 = fgetc(f);
		if (c2 == EOF) {
			fclose(f);
			FileInfo(d->dev, "[%03d/ %03d] '%s'", c, blocks,
								fname);
			DPRINT("-");
			DPRINT("E");
			DPRINT("O");
			DPRINT("F");
			DPRINT("-");
			PutIECByte(c1, 1); if (timeout) return 0;
			return 1;
		} else {
			PutIECByte(c1, 0); if (timeout) return 0;
		}
		if ((b++ % 254) == 0) {
			FileInfo(d->dev, "[%03d/ %03d] '%s'", c++, blocks,
								fname);
		}
	} while (!timeout);

	return 0;
}

static file_ops_t fs_fileops =
{
	"FS",          /* name */
	fs_check_type, /* check_type */
	fs_send_dir,   /* send_dir */
	fs_send_file,  /* send_file */
};

void register_fs(void)
{
	fileio_register(&fs_fileops);
}
