/*
 *  - common/prot.c
 *
 *  - generic serial protocol handling functions
 *
 *  This file is part of the VC 1541 Emulator.
 *  Copyright (c) 1997 Torsten Paul
 */

#include <stdio.h>

#include "vc1541.h"
#include "common.h"

int timeout; 			                   /* this is set by Wait */

void handle_CLOSE(void)
{
}

int GetIECByte(byte_t *ret)
{
    int eoi;
    byte_t b;

    eoi = 0;
    if (Wait(CLOCK, 0)) return 0;                      	       /* [$e9d5] */

    Set(DATA, 0);                                    	       /* [$e9d7] */
    if (Wait(DATA, 0)) return 0;     /* [$ff25] (patch called from $e9dc) */

    /*
     *  check for eoi
     */
    if (Wait_eoi(CLOCK, CLOCK)) {                     	       /* [$e9ee] */
	eoi = 1;
	DPRINT("*");
	Set(DATA, DATA);                              	       /* [$e9f2] */
	MicroSleep(80);                               	       /* [$e9f8] */
	Set(DATA, 0);                                 	       /* [$e9fa] */
	if (Wait(CLOCK, CLOCK)) return 0;         	       /* [$ea05] */
    }

    if (Wait(CLOCK, 0)) return 0;                   	       /* [$ea13] */
    b = (Get(DATA) >> DATA_BIT) << 0;
    if (Wait(CLOCK, CLOCK)) return 0;               	       /* [$ea22] */

    if (Wait(CLOCK, 0)) return 0;                   	       /* [$ea13] */
    b |= (Get(DATA) >> DATA_BIT) << 1;
    if (Wait(CLOCK, CLOCK)) return 0;               	       /* [$ea22] */

    if (Wait(CLOCK, 0)) return 0;                   	       /* [$ea13] */
    b |= (Get(DATA) >> DATA_BIT) << 2;
    if (Wait(CLOCK, CLOCK)) return 0;               	       /* [$ea22] */

    if (Wait(CLOCK, 0)) return 0;                   	       /* [$ea13] */
    b |= (Get(DATA) >> DATA_BIT) << 3;
    if (Wait(CLOCK, CLOCK)) return 0;               	       /* [$ea22] */

    if (Wait(CLOCK, 0)) return 0;                   	       /* [$ea13] */
    b |= (Get(DATA) >> DATA_BIT) << 4;
    if (Wait(CLOCK, CLOCK)) return 0;               	       /* [$ea22] */

    if (Wait(CLOCK, 0)) return 0;                   	       /* [$ea13] */
    b |= (Get(DATA) >> DATA_BIT) << 5;
    if (Wait(CLOCK, CLOCK)) return 0;               	       /* [$ea22] */

    if (Wait(CLOCK, 0)) return 0;                   	       /* [$ea13] */
    b |= (Get(DATA) >> DATA_BIT) << 6;
    if (Wait(CLOCK, CLOCK)) return 0;               	       /* [$ea22] */

    if (Wait(CLOCK, 0)) return 0;                   	       /* [$ea13] */
    b |= (Get(DATA) >> DATA_BIT) << 7;
    if (Wait(CLOCK, CLOCK)) return 0;               	       /* [$ea22] */

    Set(DATA, DATA);                              	       /* [$ea28] */

    *ret = ~b;                     /* all bits we got need to be inverted */

    DPRINT(ret);

    if (eoi) {
	Set(CLOCK, CLOCK);                                /* c64: [$ee0d] */
	Set(DATA, DATA);                                  /* c64: [$ee10] */
    }
#if 0
#endif
	MicroSleep(100);                               	  /* c64: [$ee0a] */

    return eoi;
}

int PutIECByte(byte_t b, int eoi)
{
    int a;
    byte_t tmp;
    int flag = 0;


    /* DELAY c64: [$ee1b] */

    DPRINT(&b);
    MicroSleep(200);

    tmp = Get(DATA);                                           /* [$e919] */
    Set(CLOCK, 0);                    			       /* [$e91f] */

    if (tmp) {                     			       /* [$e923] */
	if (Wait(DATA, 0)) return 0;			       /* [$e92d] */
	flag = eoi;
	if (eoi) DPRINT("e");
    } else {
	DPRINT("=");
	DPRINT("1");
	flag = 1;              		    /* [$e923] force eoi, why ??? */
    }
    if (flag) {
	DPRINT("=");
	DPRINT("2");
	if (Wait(DATA, 0)) return 0;			       /* [$e93f] */
	if (Wait(DATA, DATA)) return 0;			       /* [$e949] */
    }

    Set(CLOCK, CLOCK);        				       /* [$e94b] */
    if (Wait(DATA, 0)) return 0; 			       /* [$e956] */

    for (a = 0;a < 8;a++) {

	/* DELAY c64: [$ee5a] */
	MicroSleep(80);

	if (Get(DATA)) {
	    DPRINT("~");
	    return 0;					       /* [$e961] */
	}
	if (b & (1 << a)) {
	    Set(DATA, 0);     				       /* [$e973] */
	} else {
	    Set(DATA, DATA);   				       /* [$e96e] */
	}
	Set(CLOCK, 0);         				       /* [$e976] */
	MicroSleep(80);       	       /* [$e97d -> $fef3] delay for 1541 */

	/* DELAY c64: [$ee67] */

	Set(CLOCK, CLOCK);     			      /* [$e980 -> $fefb] */
	Set(DATA, 0);   		              /* [$e980 -> $fefb] */
    }
    if (Wait(DATA, DATA)) return 0;         		       /* [$e98f] */

    return flag;
}

/*
 *  [$e909]
 */
void handle_TALK(byte_t dev, byte_t sec_addr, char *iobuf)
{
	int a, c2;
	byte_t b, c1;
	FILE *f;

	switch (sec_addr) {
	case 0:
		if (iobuf[0] == '$') {
			if (device_chdir(dev, iobuf + 1)) {
				ErrorInfo("[%2d] sending directory...", dev);
				device_dir(dev);
			}
		return;
		}
		ErrorInfo("[%2d] sending file...", dev);
		device_open(dev, iobuf);
		break;
	case 1:
		ErrorInfo("handle_TALK with secondary address 1");
		break;
	case 15:
		ErrorInfo("handle_TALK command channel...");
		PutIECByte('0', 0); if (timeout) return;
		PutIECByte('0', 0); if (timeout) return;
		PutIECByte(',', 0); if (timeout) return;
		PutIECByte(' ', 0); if (timeout) return;
		PutIECByte('O', 0); if (timeout) return;
		PutIECByte('K', 0); if (timeout) return;
		PutIECByte(',', 0); if (timeout) return;
		PutIECByte('0', 0); if (timeout) return;
		PutIECByte('0', 0); if (timeout) return;
		PutIECByte(',', 0); if (timeout) return;
		PutIECByte('0', 0); if (timeout) return;
		PutIECByte('0', 0); if (timeout) return;
		PutIECByte(0x0d, 1); if (timeout) return;
		break;
	default:
		ErrorInfo("handle_TALK...");
		break;
	}
}

/*
 *  [$ea2e]
 *
 *  If iobuf != NULL we have an open command and the filename
 *  is following. The other case is a data transfer.
 *  FIXME: buffer overflow not tested!
 *
 */
void handle_LISTEN(byte_t dev, byte_t sec_addr, char *iobuf)
{
	int a;
	int eoi;
#if 0
	FILE *f;
#endif
	byte_t b;
	char name[13];
	static int nr = 0;

	switch (sec_addr) {
	case 0:
		ErrorInfo("[%2d] open...", dev);
#if 0
		sprintf(name, "out_%04d.c64", nr++);
		if (iobuf == NULL) f = fopen(name, "wb");
#endif
		a = 0;
		do {
			if (Get(ATN)) {
				DPRINT("*");
				DPRINT("A");
				DPRINT("*");
				return;
			}
			eoi = GetIECByte(&b);
			if (iobuf) {
				iobuf[a++] = b;
				iobuf[a] = '\0';
			}
			if (timeout) return;
#if 0
			if (f) fputc(b, f);
#endif
		} while (!eoi);
#if 0
		if (f) fclose(f);
#endif
		break;
	case 1:
		ErrorInfo("handle_LISTEN with secondary address 1");
		break;
	case 15:
		ErrorInfo("handle_LISTEN command channel...");
		break;
	default:
		ErrorInfo("handle_LISTEN...");
		break;
	}
}

/*
 *  [$e85b]
 */
void handle_ATN(void)
{
    byte_t b, c;
    char iobuf[C64_MAXPATH];
    byte_t mode, dev, sec_addr;

    Wait(CLOCK, 0);             			       /* [$e882] */

    timeout = 0;

    DPRINT(" ");
    DPRINT("a");

    MicroSleep(200);
    GetIECByte(&b);	         		     /* get device number */
    if (timeout) return;
    dev = b & IEC_DEVICE;
    mode = b & IEC_COMMAND;
    switch (mode) {
    case IEC_UNLISTEN:
	DPRINT("L");
	Wait(ATN, 0);
	DPRINT("A");
	return;
    case IEC_UNTALK:
	DPRINT("T");
	Wait(ATN, 0);
	DPRINT("A");
	return;
    case IEC_LISTEN:
	DPRINT("l");
	if (!device_isactive(dev)) {
		DPRINT("-");
		Set(DATA | CLOCK, 0);  			       /* [$e8ff] */
		while (Wait(ATN, 0));	 		       /* [$e907] */
		DPRINT("A");
		return;
	    }
	break;
    case IEC_TALK:
	DPRINT("t");
	if (!device_isactive(dev)) {
		DPRINT("-");
		Set(DATA | CLOCK, 0);    		       /* [$e8ff] */
		while (Wait(ATN, 0));	 		       /* [$e907] */
		DPRINT("A");
		return;
	}
	break;
    }

    MicroSleep(200);
    GetIECByte(&c);              		 /* get secondary address */

    if (timeout) return;

    switch (c & IEC_MODE) {
    case IEC_OPEN:
	Wait(ATN, 0);	         			       /* [$e907] */
	DPRINT("A");
	DPRINT("o");
	sec_addr = c & IEC_ADDRESS;
	handle_LISTEN(dev, sec_addr, iobuf);  	      /* [$e8e7 -> $ea2e] */
	DPRINT("O");
	break;
    case IEC_CLOSE:
#if 0
	Wait(ATN, 0);    				       /* [$e907] */
#endif
	DPRINT("A");
	DPRINT("c");
	handle_CLOSE();         	              /* [$e8ce -> $dac0] */
	DPRINT("C");
	break;
    case IEC_DATA:
	DPRINT("d");
	sec_addr = c & IEC_ADDRESS;
	switch (mode) {
	case IEC_LISTEN:
	    Wait(ATN, 0);	   		   	       /* [$e907] */
	    DPRINT("A");
	    handle_LISTEN(dev, sec_addr, NULL);	      /* [$e8e7 -> $ea2e] */
	    break;
	case IEC_TALK:
	    Set(DATA | CLOCK, 0); 			       /* [$e8ff] */
	    Wait(ATN, 0);	   			       /* [$e907] */
	    DPRINT("A");
	    MicroSleep(200); /* FIXME */
	    Set(DATA, 0);                		       /* [$e8f1] */
	    Set(CLOCK, CLOCK);           		       /* [$e8f4] */
	    MicroSleep(200); /* FIXME */
	    handle_TALK(dev, sec_addr, iobuf);	      /* [$e8f7 -> $e909] */
	    break;
	}
	DPRINT("D");
    }

    if (!timeout) return;
    DPRINT("*");
    DPRINT("T");
    DPRINT("*");
}
