/*
 *  common/util.c
 *
 *  - utility functions
 *
 *  This file is part of the VC 1541 Emulator.
 *  Copyright (c) 1997 Torsten Paul
 */

#include <ctype.h>
#include <dirent.h>

#include "util.h"
#include "common.h"

extern int timeout; 			           /* this is set by Wait */

int util_send_filename(word_t *addr, word_t blocks, char *filename,
							char *type)
{
	int len;
	word_t a, b;

	b = 1;
	if (blocks < 100) b++;
	if (blocks < 10) b++;
	*addr += 25 + b;
	PutIECByte(*addr, 0); if (timeout) return 0;
	PutIECByte(*addr >> 8, 0); if (timeout) return 0;
	PutIECByte(blocks, 0); if (timeout) return 0;
	PutIECByte(blocks >> 8, 0); if (timeout) return 0;
	for (a = 0;a < b;a++) {
		PutIECByte(' ', 0);
		if (timeout) return 0;
	}
	len = strlen(filename);
	PutIECByte('"', 0);
	for (a = 0;a < len;a++) {
		PutIECByte(filename[a], 0);
		if (timeout) return 0;
	}
	PutIECByte('"', 0);
	for (a = 0;a < 18 - len;a++) {
		PutIECByte(' ', 0);
		if (timeout) return 0;
	}
	len = strlen(type);
	*addr += len;
	for (a = 0;a < len;a++) {
		PutIECByte(type[a], 0);
		if (timeout) return 0;
	}
	PutIECByte(0x00, 0);
	if (timeout) return 0;

	return 1;
}

int util_glob_file(char *f1, char *f2, int ignorecase)
{
	while (1) {
		if (*f1 == '\0' && *f2 == '\0') return 1;
		if (*f1 == '\0') return 0;
		if (*f2 == '\0') {
			if (*f1 == '*') return 1;
			return 0;
		}
		if (ignorecase) {
			if (toupper(*f1) == toupper(*f2)) {
				f1++;
				f2++;
				continue;
			}
		} else {
			if (*f1 == *f2) {
				f1++;
				f2++;
				continue;
			}
		}
		if (*f1 == '*') return 1;
		return 0;
	}
}

int util_glob_dir(char *buf, char *path, char *filename)
{
	DIR *dir;
	struct dirent *ent;

	if ((dir = opendir(path)) != NULL) {
		while ((ent = readdir(dir)) != NULL) {
			if (util_glob_file(filename, ent->d_name, 1)) {
				util_path_chdir(buf, path, ent->d_name);
				closedir(dir);
				return 1;
			}
		}
		closedir(dir);
	}

	util_path_chdir(buf, path, filename);
	return 1;
}

int util_path_chdir(char *buf, char *path, char *filename)
{
	int a;

	a = strlen(path);

	if ((filename == NULL) || (filename[0] == '\0') ||
					strcmp(filename, ".") == 0) {
		strcpy(buf, path);
		return 1;
	}
	if (strcmp(filename, ".") == 0) return 1;
	if (strcmp(filename, "..") == 0) {
		strcpy(buf, path);
		for (a = strlen(buf) - 1;a > 2;a--) {
			if (buf[a] == '\\') {
				buf[a] = '\0';
				return 1;
			}
		}
		buf[3] = '\0';
		return 1;
	}

	if (a > 0 && path[a - 1] == '\\') {
		sprintf(buf, "%s%s", path, filename);
	} else {
		sprintf(buf, "%s\\%s", path, filename);
	}
	return 1;
}
