/*
 *  - common/vc1541.c
 *
 *  - main functions
 *
 *  This file is part of the VC 1541 Emulator.
 *  Copyright (c) 1997 Torsten Paul
 */

#include <stdio.h>

#ifdef LINUX

#include <errno.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/rtf.h>
#include <linux/unistd.h>
#include <linux/sched.h>
#include <asm/rt_time.h>

#endif /* LINUX */

#ifdef MSDOS

#include <dos.h>

#endif /* MSDOS */

#include "common.h"
#include "fileio.h"
#include "version.h"

#define BUFSIZE 20

int init_done = 0;
int port = -1;
long ss_timeout = 120 * 10000L; /* 2 min */
int dump_screen = 0;

char buf[BUFSIZE];

void printbuf(char *buf, int len)
{
	int a;

	printf(" :  ");
	for (a = 0;a < len;a++) {
		printf("%c", ((buf[a] < ' ') || (buf[a] > '~')) ? '.' : buf[a]);
	}
	printf("\n");
}

void print_help(char *prgname)
{
	char *n = strrchr(prgname, '\\');

	if (n == NULL) {
		n = prgname;
	} else {
		n++;
	}
	printf("\nusage: %s [options]\n\n", n);
	printf("options:\n");
	printf("  -h:               this help screen\n");
	printf("  -dump:            dump screen to file (screen.txt) on exit\n");
	printf("  -8..-11:          activate device 8..11 (default is '%s')\n",
			DEVICE_PATH_INIT);
	printf("  +8..+11 filename: activate device 8..11 and attach to 'filename'\n");
	printf("  +lpt x:           use printer port 'x' (default is '1')\n");
	printf("  +ss 0:            disable screensaver\n");
	printf("  +ss x:            activate screensaver after 'x' seconds (default is '120')\n");
}

void activate(byte_t dev, char *filename, int *a)
{
	*a = 1;
	device_activate(dev);
	if (filename == NULL) {
		printf("device %2d activated\n", dev);
	} else {
		if (device_chdir(dev, filename)) {
			printf("device %2d attached to '%s'...\n",
				dev, filename);
		} else {
			printf("warning: can't attach device %d to '%s'.\n",
				dev, filename);
			printf("device %2d attached to '%s'...\n",
				dev, DEVICE_PATH_INIT);
		}
	}
}

int useport(char *nr)
{
	int p;
	int x;
	int cable_ok;

	p = atoi(nr);
	if ((p < 1) || (p > 3)) {
		printf("error: invalid printer port 'lpt%d'.\n", p);
		exit(1);
	}

#ifdef MSDOS
	x = 2 * p + 6;

	asm push es
	asm mov  ax, 0x40
	asm mov  es, ax
	asm mov  bx, x
	asm mov  ax, es:[bx]
	asm mov  x, ax
	asm pop  es

	if (x == 0) {
		printf("error: printer port 'lpt%d' is not present.\n", p);
		exit(1);
	}

	printf("using printer port 'lpt%d' at %04xh...\n", p, x);

	cable_ok = 1;
	outportb(x, 0);
	if ((inportb(x + 1) & 0x08) != 0) cable_ok = 0;
	outportb(x, 1);
	if ((inportb(x + 1) & 0x08) == 0) cable_ok = 0;
	outportb(x, 0);
	if ((inportb(x + 1) & 0x08) != 0) cable_ok = 0;
#endif /* MSDOS */

#ifdef LINUX
	x = 0x03bc;
	cable_ok = 0;
	printf("using printer port 'lpt%d' at %04xh...\n", p, x);
#endif /* LINUX */

	port = x + 2;

	if (!cable_ok) {
		printf("warning: cable not detected.\n");
		return 0;
	} else {
		printf("cable found...\n");
	}

	return 1;
}

#ifdef LINUX

char mem[65536];
long memidx = 0;

_syscall0(int,sched_yield);
_syscall1(int,sched_getscheduler,pid_t,pid);
_syscall2(int,sched_getparam,pid_t,pid,struct sched_param *,param);
_syscall3(int,sched_setscheduler,pid_t,pid,int,policy,struct sched_param *,param);

int main_linux()
{
	FILE *f;
	fd_set rfds;
	struct timeval tv;
	int fd0, fd1, ctl;
	int n;
	int count;
	msg_t msg;
	char b;
	pid_t pid;
        struct sched_param rt_param;
	
	if ((fd0 = open("/dev/rtf0", O_RDONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf0\n");
		exit(1);
	}

	if ((fd1 = open("/dev/rtf1", O_RDONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf1\n");
		exit(1);
	}

	if ((ctl = open("/dev/rtf2", O_WRONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf2\n");
		exit(1);
	}

	if ((f = fopen("out.c64", "wb")) == NULL) {
		fprintf(stderr, "can't open out.c64\n");
		exit(1);
	}

	/* now start the tasks */
	msg.task = 0;
	msg.period = (RT_TICKS_PER_SEC * 850) / 1000000;
	msg.command = START_TASK;
	if (write(ctl, &msg, sizeof(msg)) < 0) {
		fprintf(stderr, "Can't send a command to RT-task\n");
		exit(1);
	}
	msg.task = 1;
	msg.command = START_TASK;
	msg.period = 360 * RT_TICKS_PER_SEC;
	if (write(ctl, &msg, sizeof(msg)) < 0) {
		fprintf(stderr, "Can't send a command to RT-task\n");
		exit(1);
	}


	pid = getpid();
        rt_param.sched_priority = 10;
	if (sched_setscheduler(pid, SCHED_RR, &rt_param) != 0) {
		fprintf(stderr, "warning: setscheduler() failed\n");
	}

	count = 0;
	while (1) {
		FD_ZERO(&rfds);
		FD_SET(fd0, &rfds);
		FD_SET(fd1, &rfds);
		tv.tv_sec = 0;
		tv.tv_usec = 0;

		if (select(FD_SETSIZE, &rfds, NULL, NULL, &tv) > 0) {
			if (FD_ISSET(fd0, &rfds)) {
			    n = read(fd0, &b, 1);
			    mem[memidx++] = n;
			    /* fwrite(&b, 1, 1, f); */
			    /*
			    printf("%02x ", b & 0xff);
			    buf[count] = b;
			    count++;
			    if (count == BUFSIZE) {
				count = 0;
				printbuf(buf, BUFSIZE);
			    }
			    */
			    write(fileno(f), &b, 1);
			    if (b < 0x20) b = '.';
			    write(1, &b, 1);
			    if ((memidx & 0x3f) == 0x3f) write(1, "\r", 1);
			    /*
			    printf("%c", (b < 0x20) ? '.' : b);
			    if ((memidx & 0x15) == 0) fflush(stdout);
			    */
			}
			if (FD_ISSET(fd1, &rfds)) {
				/*
				n = read(fd1, &b, 1);
				printf("\n\t\t\t\tFIFO 1: %02x (%c)\n", b & 0xff, b);
				fflush(stdout);
				*/
				break;
			}
		}
	}
	fclose(f);
	if (count > 0) printbuf(buf, count);
	printf("\n\n* terminated *\n\n");

	/* stop the tasks */
	msg.command = STOP_TASK;
	if (write(ctl, &msg, sizeof(msg)) < 0) {
		fprintf(stderr, "Can't send a command to RT-task\n");
		exit(1);
	}

	return 0;
}

#else /* LINUX */

int main_dos(void)
{
	int a;

	init_done = 1;
	clrscr();
	printf("VC 1541 Emulator V%d.%02dpl%d                               "
		"   (c) 1997 Torsten Paul",
		VERSION_MAJOR, VERSION_MINOR, PATCHLEVEL);
	for (a = FIRST_DEVICE;a < (FIRST_DEVICE + MAX_DEVICES);a++) {
		device_chdir(a, NULL);
	}
	gotoxy(1, 2);
	printf("ͻ");
	for (a = 0;a < 13;a++) {
		printf("%-79s", "");
	}
	printf("          ͹");
	printf("͹                                                                   ");
	printf("           ͼ");
	gotoxy(1, 25);
	mainloop_dos();
	gotoxy(1, 24);
	return 0;
}

#endif /* LINUX */

int main(int argc, char **argv)
{
	int a, arg;
	int sleepval;

	device_init();
	fileio_init();

	register_fs();
	register_t64();
	register_lnx();
	register_d64(); /* should be the last one because of the missing
			   file magic */

	printf("\nVC 1541 Emulator V%d.%02dpl%d\n\n",
			VERSION_MAJOR, VERSION_MINOR, PATCHLEVEL);
	printf("(c) 1997 Torsten Paul\n");
	printf("paul@os.inf.tu-dresden.de\n\n");
	a = 0;
	arg = 1;
	sleepval = 2;
	while (arg < argc) {
		if (strcmp(argv[arg], "-h") == 0) {
			print_help(argv[0]);
			exit(0);
		}
		if (strcmp(argv[arg], "-dump") == 0) {
			arg++;
			dump_screen = 1;
			continue;
		}
		if (strcmp(argv[arg], "-8") == 0) {
			arg++;
			activate(8, NULL, &a);
			continue;
		}
		if (strcmp(argv[arg], "-9") == 0) {
			arg++;
			activate(9, NULL, &a);
			continue;
		}
		if (strcmp(argv[arg], "-10") == 0) {
			arg++;
			activate(10, NULL, &a);
			continue;
		}
		if (strcmp(argv[arg], "-11") == 0) {
			arg++;
			activate(11, NULL, &a);
			continue;
		}
		if (strcmp(argv[arg], "+8") == 0) {
			arg++;
			if (arg >= argc) {
				printf("warning: missing argument for '+8'.\n");
				activate(8, NULL, &a);
				break;
			}
			activate(8, argv[arg++], &a);
			continue;
		}
		if (strcmp(argv[arg], "+9") == 0) {
			arg++;
			if (arg >= argc) {
				printf("warning: missing argument for '+9'.\n");
				activate(9, NULL, &a);
				break;
			}
			activate(9, argv[arg++], &a);
			continue;
		}
		if (strcmp(argv[arg], "+10") == 0) {
			arg++;
			if (arg >= argc) {
				printf("warning: missing argument for '+10'.\n");
				activate(10, NULL, &a);
				break;
			}
			activate(10, argv[arg++], &a);
			continue;
		}
		if (strcmp(argv[arg], "+11") == 0) {
			arg++;
			if (arg >= argc) {
				printf("warning: missing argument for '+11'.\n");
				activate(11, NULL, &a);
				break;
			}
			activate(11, argv[arg++], &a);
			continue;
		}
		if (strcmp(argv[arg], "+lpt") == 0) {
			arg++;
			if (arg >= argc) {
				printf("warning: missing argument for '+lpt'.\n");
				if (!useport(DEFAULT_PORT)) sleepval = 5;
				break;
			}
			if (!useport(argv[arg++])) sleepval = 5;
			continue;
		}
		if (strcmp(argv[arg], "+ss") == 0) {
			arg++;
			if (arg >= argc) {
				printf("warning: missing argument for '+ss'.\n");
				break;
			}
			ss_timeout = atol(argv[arg++]);
			if (ss_timeout == 0) continue;
			if ((ss_timeout < 10) || (ss_timeout > 3600)) {
				printf("warning: invalid screensaver time (available range is 10..3600).\n");
				ss_timeout = 120;
			}
			ss_timeout *= 10000L;
			continue;
		}
		printf("warning: unknown option '%s'.\n", argv[arg]);
		arg++;
	}
	DPRINT_INIT();
	if (dump_screen) {
		printf("dumping screen on exit...\n");
	}
	if (ss_timeout == 0) {
		printf("screensaver disabled...\n");
	} else {
		printf("screensaver after %ld seconds...\n",
							ss_timeout / 10000L);
	}
	if (port == -1) if (!useport(DEFAULT_PORT)) sleepval = 5;
	if (a == 0) {
		printf("\nwarning: no device activated.\n");
		printf("use -h for help.\n");
		sleepval = 5;
	}
	sleep(sleepval);

#ifdef LINUX
	main_linux();
#endif /* LINUX */
#ifdef MSDOS
	main_dos();
#endif /* MSDOS */

	return 0;
}
