/*
 *  - dos/io_dos.c
 *
 *  - DOS specific I/O and timing functions
 *
 *  This file is part of the VC 1541 Emulator.
 *  Copyright (c) 1997 Torsten Paul
 */

#include <time.h>
#include <stdarg.h>
#include <stdlib.h>

#include "common.h"
#include "fileio.h"

#define DISPLAY_OFFSET 322

extern int timeout;
extern int init_done;
extern int port;

unsigned short videoseg = 0xb800;
static unsigned char  col      = 0x0c;
static unsigned char  colxor   = 0x0f;
static unsigned short idx      = 0;
static unsigned short column   = 78;
static unsigned short count    = 0x400;

static char screenbuf[4000];

void Set(byte_t bit, byte_t val)
{
	static byte_t v = 0xc4;

	asm	mov	dx, port
	asm	mov	ah, bit
	asm	xor	ah, 0xff
	asm	mov	al, v
	asm	and	al, ah
	asm	or	al, val
	asm	out	dx, al
	asm	mov	v,  al
}


byte_t Get(byte_t bit)
{
	byte_t val;

	asm	mov	dx, port

	__l0:
	asm 	in	al, dx
	asm	mov	ah, al
	asm	in	al, dx
	asm	cmp	al, ah
	asm	jne	 __l0
	asm	and	al, bit
	asm	mov	val, al

		return val;
}

void MicroSleep(byte_t usec)
{
	asm	mov	al, 0x92
	asm	out	0x43, al
	asm	mov	al, usec
	asm	out	0x42, al

	asm	in	al, 0x61
	asm	mov	ah, al
	asm	or	al, 1
	asm	out	0x61, al
	asm	mov	al, ah
	asm	out	0x61, al

	__l0:
	asm	in	al, 0x42
	asm	cmp	al, 250
	asm	jnl	__l0
}

/*
 *  the PIT uses a frequency of 1193181 Hz
 *  -> one timer tick every 0.838 micro seconds
 *  -> 100 micro seconds = 120 timer ticks
 */
byte_t Wait_eoi(byte_t bit, byte_t val)
{
	word_t count = IO_TIMEOUT1;

	asm	mov	dx, port
	asm	mov	bx, 10
	__l0:
	asm	in	al, dx
	asm	mov	ah, al
	asm	in	al, dx
	asm	cmp	al, ah
	asm	jne	__l0

	asm	and	al, bit
	asm	cmp	al, val
	asm	jnz	__l2
	__l1:
		return 0;
	__l2:
	asm	dec	bx
	asm	jnz	__l0
	__l3:
	asm	mov	al, 0x92
	asm	out	0x43, al
	asm	mov	al, 120
	asm	out	0x42, al

	asm	in	al, 0x61
	asm	or	al, 1
	asm	out	0x61, al
	asm	and	al, 0xfe
	asm	out	0x61, al

	__l4:
	asm	in	al, dx
	asm	mov	ah, al
	asm	in	al, dx
	asm	cmp	al, ah
	asm	jne	__l4
	asm	and	al, bit
	asm	cmp	al, val
	asm	jz	__l1

	asm	in	al, 0x42
	asm	cmp	al, 250
	asm	jl	__l4

	asm	dec	count
	asm	jnz	__l3

		return 1;
}

byte_t Wait(byte_t bit, byte_t val)
{
	word_t count = IO_TIMEOUT2;

	asm	mov	dx, port
	asm	mov	bx, 200
	__l0:
	asm	in	al, dx
	asm	mov	ah, al
	asm	in	al, dx
	asm	cmp	al, ah
	asm	jne	__l0

	asm	and	al, bit
	asm	cmp	al, val
	asm	jnz	__l2
	__l1:
		return 0;
	__l2:
	asm	dec	bx
	asm	jnz	__l0
	__l3:
	asm	mov	al, 0x92
	asm	out	0x43, al
	asm	mov	al, 120
	asm	out	0x42, al

	asm	in	al, 0x61
	asm	or	al, 1
	asm	out	0x61, al
	asm	and	al, 0xfe
	asm	out	0x61, al
	__l4:
	asm	in	al, dx
	asm	mov	ah, al
	asm	in	al, dx
	asm	cmp	al, ah
	asm	jne	__l4
	asm	and	al, bit
	asm	cmp	al, val
	asm	jz	__l1

	asm	in	al, 0x42
	asm	cmp	al, 250
	asm	jl	__l4

	asm	dec	count
	asm	jnz	__l3

		timeout = 1;
		return 1;
}

void DPRINT1(unsigned char v)
{
	asm {
		push es
		mov  ax, videoseg
		mov  es, ax
		mov  bx, idx
		mov  al, v
		mov  es:[bx+DISPLAY_OFFSET], al
		inc  bx
		mov  al, col
		mov  es:[bx+DISPLAY_OFFSET],al
		inc  bx

		dec  column
		jnz  __l1

		/* end of line */
		mov  cx, 78
		mov  column, cx
		add  bx, 4
	}
	__l1:
	asm {
		dec  count
		jnz  __l2

		/* end of screen */
		mov  cx, 78
		mov  column, cx
		mov  cx, 0x400
		mov  count, cx
		mov  al, colxor
		xor  col, al
		xor  bx, bx
		xor  cx, cx

	}
	__l2:
	asm {
		mov  idx, bx
		pop  es
	}
}

void Idle(void)
{
	static word_t a = 0;
	static char idlestr[] = "|/-\\";
	byte_t c = idlestr[a >> 14];

	a += 32;
	asm {
		push es
		mov  ax, videoseg
		mov  es, ax
		mov  al, c
		mov  bx, idx
		mov  es:[bx+DISPLAY_OFFSET], al
		inc  bx
		mov  al,es:[bx+DISPLAY_OFFSET]
		and  al, 0x03
		or   al, 0x04
		mov  es:[bx+DISPLAY_OFFSET], al
		pop  es
	}
}

#define STARS 100
void ScreenSaver(int on)
{
	char c;
	int a, i;
	time_t t;
	static int count = 0;
	static int active = 0;
	static int val[STARS];
	static int idx[STARS];
	char *str = "--++***######***++-- ";

	if (!on) {
		if (active) {
			asm	push	es
			asm	mov	ax, videoseg
			asm	mov	es, ax
			asm	xor	bx, bx
			__l0:
			asm	mov	al, byte ptr screenbuf[bx]
			asm	mov	es:[bx], al
			asm	inc	bx
			asm	cmp	bx, 4000
			asm	jne	__l0
			asm	pop	es
		}
		active = 0;
		return;
	}
	if (!active) {
		for (a = 0;a < STARS;a++) {
			val[a] = a % strlen(str);
		}
		srand((unsigned) time(&t));
		asm	push	es
		asm	mov	ax, videoseg
		asm	mov	es, ax
		asm	xor	bx, bx
		__l1:
		asm	mov	al, es:[bx]
		asm	mov	byte ptr screenbuf[bx], al
		asm	inc	bx
		asm	cmp	bx, 4000
		asm	jne	__l1
		asm	pop	es
	}
	active = 1;

	if (++count < 200) return;
	count = 0;

	for (a = 0;a < STARS;a++) {
		if (val[a] == 0) {
			idx[a] = (rand() % 4000) & ~1;
		}
		c = str[val[a]];
		if (++val[a] >= strlen(str)) val[a] = 0;
		i = idx[a];

		asm	push	es
		asm	mov	ax, videoseg
		asm	mov	es, ax
		asm	mov	bx, i
		asm	mov	al, c
		asm	mov	ah, 0x07
		asm	mov	word ptr es:[bx], ax
		asm	pop	es
	}
}

void DeviceInfo(byte_t dev, char *fmt, ...)
{
	int x, y;
	va_list ap;

	if (init_done) {
		x = wherex();
		y = wherey();
		va_start(ap, fmt);
		gotoxy(1, dev - FIRST_DEVICE + 21);
		printf("%79s", "");
		gotoxy(1, dev - FIRST_DEVICE + 21);
		vprintf(fmt, ap);
		va_end(ap);
		gotoxy(x, y);
	}
}

void FileInfo(byte_t dev, char *fmt, ...)
{
	int x, y;
	va_list ap;
	char buf[80];

	if (init_done) {
		x = wherex();
		y = wherey();
		va_start(ap, fmt);
		vsprintf(buf, fmt, ap);
		va_end(ap);
		gotoxy(1, 19);
		printf("(%2d) %-75.75s", dev, buf);
		gotoxy(x, y);
	}
}

void ErrorInfo(char *fmt, ...)
{
	int x, y;
	va_list ap;

	if (init_done) {
		x = wherex();
		y = wherey();
		gotoxy(14, 17);
		printf("%65s", " ");
		gotoxy(14, 17);
		va_start(ap, fmt);
		vprintf(fmt, ap);
		va_end(ap);
		gotoxy(x, y);
	}
}

void DPRINT_INIT(void)
{
	if (inportb(0x3cc) & 1) {
		videoseg = 0xb800;
		col      = 0x03;
		colxor   = 0x0f;
		printf("using color display...\n");
	} else {
		videoseg = 0xb000;
		col      = 0x03;
		colxor   = 0x7f;
		printf("using monochrome display...\n");
	}
}
