/*
 *  - include/common.h
 *
 *  This file is part of the VC 1541 Emulator.
 *  Copyright (c) 1997 Torsten Paul
 */

#ifndef __common_h
#define __common_h

#ifdef LINUX
#include <linux/rt_sched.h>
#include <linux/rtf.h>
#include <asm/system.h>
#else /* LINUX */
#endif /* LINUX */


/*
 ************
 *  COMMON
 ************
 */

#define DEFAULT_PORT "1"
#define C64_MAXPATH 160

typedef unsigned char byte_t;
typedef unsigned short word_t;
typedef unsigned long dword_t;

/*
 ************
 *  LINUX
 ************
 */

#ifdef LINUX

#define START_TASK	1
#define STOP_TASK	2

struct _msg_struct {
	int task;
	int period;
	int command;
};
typedef struct _msg_struct msg_t;

#define IO_TIMEOUT   ((RT_TICKS_PER_SEC * 1200) / 1000000)
#define DPRINT(x)    rtf_put(0, x, 1);
/*
 *  This has to be a macro to prevent calls to divide function. Because of
 *  this it should only called with literal values
 */
#define MicroSleep(usec) \
	do { \
		RTIME t = rt_get_time() + ((RT_TICKS_PER_SEC * usec) / 1000000); \
		while (t > rt_get_time()); \
	} while (0);

#endif /* LINUX */

/*
 ************
 *  DOS
 ************
 */

#ifdef MSDOS

#define cli()	       asm cli;
#define sti()          asm sti;
#define rt_task_wait()
#define IO_TIMEOUT1    20
#define IO_TIMEOUT2    50000
#define DPRINT(x)      DPRINT1(*x);
void MicroSleep(byte_t usec);

#endif /* MSDOS */

void   Set(byte_t bit, byte_t val);
byte_t Get(byte_t bit);
byte_t Wait(byte_t bit, byte_t val);
byte_t Wait_eoi(byte_t bit, byte_t val);
void   handle_ATN(void);

#endif /* __common_h */
