/*
 *  - include/fileio.h
 *
 *  This file is part of the VC 1541 Emulator.
 *  Copyright (c) 1997 Torsten Paul
 */

#ifndef __fileio_h
#define __fileio_h

#ifdef MSDOS
#include <dir.h>
#include <sys\stat.h>
#endif /* MSDOS */

#ifdef LINUX
#include <sys/stat.h>
#endif /* LINUX */

#include "common.h"

#define MAX_FILE_OPS 10
#define MAX_DEVICES  4
#define FIRST_DEVICE 8

#ifdef MSDOS
#define DEVICE_PATH_INIT "c:\\"
#define DIR_SEPARATOR '\\'
#define PATH_MAX MAXPATH
#endif /* MSDOS */
#ifdef LINUX
#define DEVICE_PATH_INIT "/"
#define DIR_SEPARATOR '/'
#endif /* LINUX */

struct device;

struct file_ops
{
	char *name;
	int (*check_type)(char *path, struct stat *st);
	int (*send_dir)(struct device *d);
	int (*send_file)(struct device *d, char *filename);
};

struct device
{
	byte_t           dev;
	char             path[PATH_MAX];
	struct file_ops *file_ops;
};

typedef struct file_ops file_ops_t;
typedef struct device device_t;

void		fileio_init(void);
void		fileio_register(file_ops_t *ops);
file_ops_t *	fileio_check_type(char *path, struct stat *st);

void		device_init(void);
void		device_activate(byte_t dev);
int		device_isactive(byte_t dev);
int		device_chdir(byte_t dev, char *filename);
int		device_dir(byte_t dev);
int		device_open(byte_t dev, char *filename);

#endif /* __fileio_h */
